// Weather.c
//
#include "resource.h"

unsigned char readbuff[64] absolute 0x500;
unsigned char writebuff[64] absolute 0x540;

unsigned int hour;
unsigned int minute;
unsigned int second;

unsigned char strCity[64];
unsigned char strCondition[64];
unsigned char strTemp[64];
unsigned char strHumidity[64];
unsigned char strWindCondition[64];

char tmp[20];

 // TFT Connections
//
char TFT_DataPort at PORTJ;
char TFT_DataPort_Direction at TRISJ;

sbit TFT_RST at LATD3_bit;
sbit TFT_BLED at LATH5_bit;
sbit TFT_RS at LATH6_bit;
sbit TFT_CS at LATG3_bit;
sbit TFT_RD at LATH1_bit;
sbit TFT_WR at LATH2_bit;
sbit TFT_RST_Direction at TRISD3_bit;
sbit TFT_BLED_Direction at TRISH5_bit;
sbit TFT_RS_Direction at TRISH6_bit;
sbit TFT_CS_Direction at TRISG3_bit;
sbit TFT_RD_Direction at TRISH1_bit;
sbit TFT_WR_Direction at TRISH2_bit;

// Touch Panel Connections
//
sbit DriveX_Left at LATA0_bit;
sbit DriveX_Right at LATD7_bit;
sbit DriveY_Up at LATA1_bit;
sbit DriveY_Down at LATD4_bit;
sbit DriveX_Left_Direction at TRISA0_bit;
sbit DriveX_Right_Direction at TRISD7_bit;
sbit DriveY_Up_Direction at TRISA1_bit;
sbit DriveY_Down_Direction at TRISD4_bit;

const X_HOUR = 240;
const Y_HOUR = 200;

const X_CITY = 20;
const Y_CITY = 80;

const X_CONDITION = 20;
const Y_CONDITION = 100;

const X_TEMP = 260;
const Y_TEMP = 20;

const X_HUMIDITY = 20;
const Y_HUMIDITY = 120;

const X_WIND_CONDITION = 20;
const Y_WIND_CONDITION = 140;

const X_IMAGE = 20;
const Y_IMAGE = 20;

void interrupt()
{
   USB_Interrupt_Proc();

   if (TMR0IF_bit)
   {
     TMR0H = 0xB;
     TMR0L = 0xDC;
     TMR0IF_bit = 0;
   }
   
}

// Initialize
//
void Init()
{

  PLLEN_bit = 1;
  Delay_ms(150);

  ANCON0 = 0xF0;
  ADSHR_bit  = 1;
  ANCON1  = 0xFF;
  ADSHR_bit  = 0;

  CM1CON  = 0;
  CM2CON  = 0;

  T0CON.TMR0ON = 1;
  T0CON.T08BIT = 0;
  T0CON.T0CS   = 0;
  T0CON.T0SE   = 0;
  T0CON.PSA    = 0;
  T0CON.T0PS2  = 1;
  T0CON.T0PS1  = 0;
  T0CON.T0PS0  = 0;
  TMR0H = 0xB;
  TMR0L = 0xDC;

  TMR0IF_bit = 0;
  TMR0IE_bit = 1;
  INTCON = 0xA0;

  TRISF7_bit = 0;
  LATF7_bit = 1;

  ADC_Init();
  TFT_Init(320, 240);
  TP_TFT_Init(320, 240, 0, 1);
  TP_TFT_Set_ADC_Threshold(900);

}

// Display Screen
//
void DisplayScreen()
{
  TFT_Fill_Screen(CL_BLACK);
  TFT_Set_Font(Tahoma12x15, CL_WHITE, FO_HORIZONTAL);
  
  TFT_Set_Brush(1, TFT_RGBToColor16bit(78, 132, 166), 0, LEFT_TO_RIGHT, CL_BLACK, CL_BLACK);
  TFT_Set_Pen(TFT_RGBToColor16bit(255,255,255), 1);
  TFT_Rectangle_Round_Edges(0, 0, 319, 239, 20);
  TFT_Set_Pen(TFT_RGBToColor16bit(78, 132, 166), 1);
}

// Display Hour
//
void DisplayHour()
{
  TFT_Rectangle(X_HOUR, Y_HOUR, 318, Y_HOUR+16);
  TFT_Write_Text(tmp, X_HOUR, Y_HOUR);
}

// Display City
//
void DisplayCity()
{
  TFT_Rectangle(X_CITY, Y_CITY, 318, Y_CITY+16);
  TFT_Write_Text(strCity, X_CITY, Y_CITY);
}

// Display Condition
//
void DisplayCondition()
{
  TFT_Rectangle(X_CONDITION, Y_CONDITION, 318, Y_CONDITION+16);
  TFT_Write_Text(strCondition, X_CONDITION, Y_CONDITION);
}

// Display Temp
//
void DisplayTemp()
{
  TFT_Rectangle(X_TEMP, Y_TEMP, 318, Y_TEMP+16);
  TFT_Write_Text(strTemp, X_TEMP, Y_TEMP);
}

// Display Humidity
//
void DisplayHumidity()
{
  TFT_Rectangle(X_HUMIDITY, Y_HUMIDITY, 318, Y_HUMIDITY+16);
  TFT_Write_Text(strHumidity, X_HUMIDITY, Y_HUMIDITY);
}

// Display Wind Condition
//
void DisplayWindCondition()
{
  TFT_Rectangle(X_HUMIDITY, Y_WIND_CONDITION, 318, Y_WIND_CONDITION+16);
  TFT_Write_Text(strWindCondition, X_WIND_CONDITION, Y_WIND_CONDITION);
}
// Entry Point
//
void main(void)
{
  int i,j,n;

  Init();
  Delay_100ms();

  DisplayScreen();
 
  HID_Enable(&readbuff,&writebuff);
  
  while(1)
  {
    if ( HID_Read())
    {

      if ( readbuff[0] == '1' )
      {
       hour = ( readBuff[1] - '0' ) * 10 + ( readBuff[2] - '0' );
        minute = ( readBuff[4] - '0' ) * 10 + ( readBuff[5] - '0' );
        second = ( readBuff[7] - '0' ) * 10 + ( readBuff[8] - '0' );
        sprintf(tmp, "%2.2d:%2.2d:%2.2d", hour, minute, second);
        DisplayHour();
      }
      else if ( readbuff[0] == '2' )
      {
        j = 1;
        for (i=0; i<63; i++)
        {
          strCity[i] = readbuff[j];
          j++;
        }
        DisplayCity();
      }
      else if ( readbuff[0] == '3' )
      {
        j = 1;
        for (i=0; i<63; i++)
        {
          strCondition[i] = readbuff[j];
          j++;
        }
        DisplayCondition();
      }
      else if ( readbuff[0] == '4' )
      {
        j = 1;
        for (i=0; i<63; i++)
        {
          strTemp[i] = readbuff[j];
          j++;
        }
        DisplayTemp();
      }
      else if ( readbuff[0] == '5' )
      {
        j = 1;
        for (i=0; i<63; i++)
        {
          strHumidity[i] = readbuff[j];
          j++;
        }
        DisplayHumidity();
      }
      else if ( readbuff[0] == '6' )
      {
        j = 1;
        for (i=0; i<63; i++)
        {
          strWindCondition[i] = readbuff[j];
          j++;
        }
        DisplayWindCondition();
      }
      else if ( readbuff[0] == 'P' )
      {
        n = readbuff[1];
        j=2;
        for (i=0; i<n; i++)
        {
          TFT_Dot(readbuff[j+1]+X_IMAGE, readbuff[j]+Y_IMAGE, TFT_RGBToColor16bit(readbuff[j+2],readbuff[j+3],readbuff[j+4]));
          j += 5;
        }
      }
    }
  }
}