﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Net;
using System.Xml;

namespace Weather
{
  public partial class frmMain : Form
  {

    Boolean bCelcius = true;

    int oldSecond;
    int cptWeather;

    string strCityChoice = "Nalinnes";
    string strLanguage = "fr";

    string strCity;
    string strCondition;
    string strIcon;
    string strTemp;
    string strHumidity;
    string strWindCondition;

    string oldIcon = "";

    public frmMain()
    {
      InitializeComponent();
    }

    private void OnExit(object sender, EventArgs e)
    {
      Application.Exit();
    }

    private void OnLoad(object sender, EventArgs e)
    {
      oldSecond = -1;
      cptWeather = 1;
      menuCelcius.Checked = true;
      menuFahrenheit.Checked = false;
    }

    protected override void OnHandleCreated(EventArgs e)
    {
      base.OnHandleCreated(e);
      usbComponent.RegisterHandle(Handle);
      usbComponent.CheckDevicePresent();
    }

    protected override void WndProc(ref Message m)
    {
      usbComponent.ParseMessages(ref m);
      base.WndProc(ref m);
    }

    private void OnDataReceived(object sender, UsbLibrary.DataReceivedEventArgs args)
    {

    }

    private void OnDataSend(object sender, UsbLibrary.DataSendEventArgs args)
    {

    }

    private void OnDeviceArrived(object sender, EventArgs e)
    {
      notifyIcon.Text = "Usb App : Device Connected";
      notifyIcon.BalloonTipText = "Device Connected";
      notifyIcon.ShowBalloonTip(2000);
      cptWeather = 2;
      oldIcon = "";
      timer.Enabled = true;
    }

    private void OnDeviceRemoved(object sender, EventArgs e)
    {
      notifyIcon.Text = "Usb App : Device Not Connected";
      notifyIcon.BalloonTipText = "Device Not Connected";
      notifyIcon.ShowBalloonTip(2000);
      timer.Enabled = false;
    }

    private void Send(string text)
    {
     
      if (text.Length > 64)
      {
        text = text.Substring(0, 64);
      }

      while (text.Length < 64)
        text += Convert.ToChar(0);
      byte[] data = new byte[65];
      for (int i = 0; i < 64; i++)
      {
        data[i + 1] = (byte)Convert.ToByte(text[i]);
      }
      data[0] = 0;

      if (usbComponent.SpecifiedDevice != null)
      {
        usbComponent.SpecifiedDevice.SendData(data);
      }
    }

    private void GetWeather()
    {
      string strUrl = "http://www.google.com/ig/api?weather=" + strCityChoice + "&hl=" + strLanguage;
      string strName;

      WebResponse objResponse;
      WebRequest objRequest = HttpWebRequest.Create(strUrl);
      objResponse = objRequest.GetResponse();
      StreamReader sr = new StreamReader(objResponse.GetResponseStream(),Encoding.GetEncoding("ISO8859-1"));
      XmlReader reader = XmlReader.Create(sr);

      Boolean bForecastInformation = false;
      Boolean bCurrentConditions = false;

      strCity = "";
      strCondition = "";
      strIcon = "";
      strTemp = "";
      strHumidity = "";
      strWindCondition = "";

      while (reader.Read())
      {
        switch (reader.NodeType)
        {
          case XmlNodeType.Element:
            strName = reader.Name.ToLower();

            if (bForecastInformation)
            {
              if (strName == "city")
              {
                strCity = reader.GetAttribute("data");
              }
            }
            else if (bCurrentConditions)
            {
              if (strName == "condition")
              {
                strCondition = reader.GetAttribute("data");
              }
              else if (strName == "temp_c")
              {
                if (bCelcius )
                  strTemp = reader.GetAttribute("data") + " °C";
              }
              else if (strName == "temp_f") 
              {
                if ( !bCelcius)
                  strTemp = reader.GetAttribute("data") + " °F";
              }
              else if (strName == "icon")
              {
                strIcon = reader.GetAttribute("data");
              }
              else if (strName == "humidity")
              {
                strHumidity = reader.GetAttribute("data");
              }
              else if (strName == "wind_condition")
              {
                strWindCondition = reader.GetAttribute("data");
              }
            }
            else if (strName == "forecast_information")
            {
              bForecastInformation = true;
            }
            else if (strName == "current_conditions")
            {
              bCurrentConditions = true;
            }
            break;
          case XmlNodeType.Text:
            break;
          case XmlNodeType.EndElement:
            strName = reader.Name.ToLower();
            if (strName == "forecast_information")
            {
              bForecastInformation = false;
            }
            else if (strName == "current_conditions")
            {
              bCurrentConditions = false;
            }
            break;
        }
      }
    }

    private void SendIcon()
    {
      if (strIcon == "")
        return;

      if (strIcon != oldIcon)
      {
        oldIcon = strIcon;

        pictureBox.Load("http://www.google.com" + strIcon);

        Bitmap bmp = new Bitmap(pictureBox.Image);
        if (bmp == null)
          return;

        if ((bmp.Width != 40) || (bmp.Height != 40))
          return;

        Rectangle rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
        System.Drawing.Imaging.BitmapData bmpData = bmp.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadWrite, bmp.PixelFormat);

        IntPtr ptr = bmpData.Scan0;

        int bytes = Math.Abs(bmpData.Stride) * bmp.Height;
        byte[] rgbValues = new byte[bytes];

        System.Runtime.InteropServices.Marshal.Copy(ptr, rgbValues, 0, bytes);

        bmp.UnlockBits(bmpData);

        byte[] data = new byte[65];
        for (int i = 0; i < 65; i++)
        {
         data[i] = 0;
        }
        data[1] = Convert.ToByte('P');
        int idx = 0;
        int n = 0;
        for (int r=0; r<bmp.Width; r++)
        {
          for (int c = 0; c < bmp.Height; c++)
          {
            data[n*5+3] = (byte)r;
            data[n*5+4] = (byte)c;
            data[n*5+5] = rgbValues[idx++];
            data[n*5+6] = rgbValues[idx++];
            data[n*5+7] = rgbValues[idx++];
            idx++;
            n++;
            if (n == 12)
            {
              data[2] = (byte)n;
              if (usbComponent.SpecifiedDevice != null)
              {
                usbComponent.SpecifiedDevice.SendData(data);
              }
              n = 0;
            }
          }
        }
        if (n > 0)
        {
          data[2] = (byte)n;
          if (usbComponent.SpecifiedDevice != null)
          {
            usbComponent.SpecifiedDevice.SendData(data);
          }
        }
      }
    }

    private void OnTimer(object sender, EventArgs e)
    {
      DateTime dt = DateTime.Now;
      string text;

      if (oldSecond == dt.Second)
        return;
      oldSecond = dt.Second;

      cptWeather--;
      if ( cptWeather < 1 )
      {
        cptWeather = 10;
        GetWeather();
        text = "2" + strCity;
        Send(text);  
        text = "3" + strCondition;
        Send(text);
        text = "4" + strTemp;
        Send(text);
        text = "5" + strHumidity;
        Send(text);
        text = "6" + strWindCondition;
        Send(text);
        SendIcon();
      }
      text = "1" + dt.Hour.ToString("d2") + ":" + dt.Minute.ToString("d2") + ":" + dt.Second.ToString("d2");
      Send(text);
    }

    private void OnCity(object sender, EventArgs e)
    {
      frmConfiguration dlg = new frmConfiguration();
      dlg.strCity = strCityChoice;
      dlg.strLanguage = strLanguage;
      dlg.ShowDialog();
      if (dlg.strCity != "")
      {
        strCityChoice = dlg.strCity;
        strLanguage = dlg.strLanguage;
        cptWeather = 1;
      }

    }

    private void fahrenToolStripMenuItem_Click(object sender, EventArgs e)
    {
      menuCelcius.Checked = false;
      menuFahrenheit.Checked = true;
      bCelcius = false;
      cptWeather = 1;
    }

    private void celsiusToolStripMenuItem_Click(object sender, EventArgs e)
    {
      menuCelcius.Checked = true;
      menuFahrenheit.Checked = false;
      bCelcius = true;
      cptWeather = 1;
    }

  }
}
